﻿using UnityEngine;

[ExecuteInEditMode]
public class DrawBillboards : MonoBehaviour
{
    public Texture tex;
    public Material mat;

    public Mesh quadMesh;

    private ComputeBuffer quadUVs;
    private ComputeBuffer quadIndices;
    private ComputeBuffer quadVertices;
    private ComputeBuffer instanceColors;
    private ComputeBuffer instancePositions;

    public int NumInstances;
    public int SpawnRegionSize;

    void OnRenderObject()
    {
        if (Camera.current == null) return;

        CreateResources();

        mat.SetVector("_CameraUp", Camera.current.transform.up);
        mat.SetVector("_CameraRight", Camera.current.transform.right);

        mat.SetTexture("_Texture", tex);
        mat.SetBuffer("_QuadUVs", quadUVs);
        mat.SetBuffer("_QuadIndices", quadIndices);
        mat.SetBuffer("_QuadVertices", quadVertices);
        mat.SetBuffer("_InstanceColors", instanceColors);
        mat.SetBuffer("_InstancePositions", instancePositions);        

        mat.SetPass(0);
        Graphics.DrawProcedural(MeshTopology.Triangles, quadIndices.count, NumInstances);
    }

    void CreateResources()
    {
        if (quadUVs == null)
        {
            var uvs = quadMesh.uv;
            quadUVs = new ComputeBuffer(uvs.Length, sizeof(float) * 2);
            quadUVs.SetData(uvs);
        }

        if (quadIndices == null)
        {
            var indices = quadMesh.triangles;
            quadIndices = new ComputeBuffer(indices.Length, sizeof(float) * 1);
            quadIndices.SetData(indices);
        }

        if (quadVertices == null)
        {
            var normals = quadMesh.vertices;
            quadVertices = new ComputeBuffer(normals.Length, sizeof(float) * 3);
            quadVertices.SetData(normals);
        }

        if (instanceColors == null)
        {
            var colors = new Color[NumInstances];
            for (int i = 0; i < NumInstances; i++)
            {
                colors[i] = new Color(Random.value, Random.value, Random.value);
            }

            instanceColors = new ComputeBuffer(NumInstances, sizeof(float) * 4);
            instanceColors.SetData(colors);
        }

        if (instancePositions == null)
        {
            var positions = new Vector3[NumInstances];
            for (int i = 0; i < NumInstances; i++)
            {
                positions[i] = Random.insideUnitSphere * SpawnRegionSize;
            }

            instancePositions = new ComputeBuffer(NumInstances, sizeof(float) * 3);
            instancePositions.SetData(positions);
        }
    }

    void OnDisable()
    {
        if (quadUVs != null)
        {
            quadUVs.Release();
            quadUVs = null;
        }
        if (quadIndices != null)
        {
            quadIndices.Release();
            quadIndices = null;
        }

        if (quadVertices != null)
        {
            quadVertices.Release();
            quadVertices = null;
        }

        if (instanceColors != null)
        {
            instanceColors.Release();
            instanceColors = null;
        }

        if (instancePositions != null)
        {
            instancePositions.Release();
            instancePositions = null;
        }
    }
}
