﻿using UnityEngine;

[ExecuteInEditMode]
public class DrawInstanced : MonoBehaviour
{
    public Mesh mesh;
    public Texture tex;
    public Material mat;

    public int NumInstances;
    public int SpawnRegionSize;

    private ComputeBuffer meshUVs;
    private ComputeBuffer meshIndices;
    private ComputeBuffer meshNormals;
    private ComputeBuffer meshVertices;
    private ComputeBuffer instanceColors;
    private ComputeBuffer instanceMatrices;

    void OnRenderObject()
    {
        if (Camera.current == null) return;

        CreateResources();

        mat.SetTexture("_Texture", tex);
        mat.SetBuffer("_MeshUVs", meshUVs);
        mat.SetBuffer("_MeshIndices", meshIndices);
        mat.SetBuffer("_MeshNormals", meshNormals);
        mat.SetBuffer("_MeshVertices", meshVertices);
        mat.SetBuffer("_InstanceColors", instanceColors);
        mat.SetBuffer("_InstanceMatrices", instanceMatrices);

        mat.SetPass(0);
        Graphics.DrawProcedural(MeshTopology.Triangles, meshIndices.count, instanceMatrices.count);
    }

    void CreateResources()
    {
        if (meshUVs == null)
        {
            var uvs = mesh.uv;
            meshUVs = new ComputeBuffer(uvs.Length, sizeof(float) * 2);
            meshUVs.SetData(uvs);
        }

        if (meshIndices == null)
        {
            var indices = mesh.triangles;
            meshIndices = new ComputeBuffer(indices.Length, sizeof(float) * 1);
            meshIndices.SetData(indices);
        }

        if (meshNormals == null)
        {
            var normals = mesh.normals;
            meshNormals = new ComputeBuffer(normals.Length, sizeof(float) * 3);
            meshNormals.SetData(normals);
        }

        if (meshVertices == null)
        {
            var vertices = mesh.vertices;
            meshVertices = new ComputeBuffer(vertices.Length, sizeof(float) * 3);
            meshVertices.SetData(vertices);
        }

        if (instanceColors == null)
        {
            var colors = new Color[NumInstances];
            for (int i = 0; i < NumInstances; i++)
            {
                colors[i] = new Color(Random.value, Random.value, Random.value);
            }

            instanceColors = new ComputeBuffer(NumInstances, sizeof(float) * 4);
            instanceColors.SetData(colors);
        }

        if (instanceMatrices == null)
        {
            var matrices = new Matrix4x4[NumInstances];
            for (int i = 0; i < NumInstances; i++)
            {
                matrices[i] = Matrix4x4.TRS(Random.insideUnitSphere * SpawnRegionSize, Random.rotation, Vector3.one);
            }

            instanceMatrices = new ComputeBuffer(NumInstances, sizeof(float) * 16);
            instanceMatrices.SetData(matrices);
        }
    }

    void OnDisable()
    {
        if (meshUVs != null)
        {
            meshUVs.Release();
            meshUVs = null;
        }
        if (meshIndices != null)
        {
            meshIndices.Release();
            meshIndices = null;
        }

        if (meshNormals != null)
        {
            meshNormals.Release();
            meshNormals = null;
        }

        if (meshVertices != null)
        {
            meshVertices.Release();
            meshVertices = null;
        }

        if (instanceColors != null)
        {
            instanceColors.Release();
            instanceColors = null;
        }

        if (instanceMatrices != null)
        {
            instanceMatrices.Release();
            instanceMatrices = null;
        }
    }
}
